import { useState, useEffect, useLayoutEffect, useMemo } from 'react';
import Mousetrap from 'mousetrap';

/**
 * Creates a global state singleton.
 */
const createStateHook = () => {
    let keys = [];
    const addKeys = (nextKeys) => {
        keys = [...keys, ...nextKeys];
        nextKeys.forEach((k) => {
            var _a;
            if (!k.disabled) {
                const mousetrap = ((_a = k.ref) === null || _a === void 0 ? void 0 : _a.current) ? Mousetrap(k.ref.current) : Mousetrap;
                mousetrap.bind(k.keys, k.callback, k.action);
            }
        });
    };
    const removeKeys = (nextKeys) => {
        keys = keys.filter((k) => !nextKeys.includes(k));
        nextKeys.forEach(s => Mousetrap.unbind(s.keys, s.action));
    };
    return () => {
        const [state, setState] = useState([]);
        useEffect(() => {
            setState(keys);
        }, []);
        return [state, addKeys, removeKeys];
    };
};
const useHotkeyState = createStateHook();

const useHotkeys = (shortcuts) => {
    const [keys, addKeys, removeKeys] = useHotkeyState();
    useLayoutEffect(() => {
        if (shortcuts) {
            addKeys(shortcuts);
        }
        return () => {
            if (shortcuts) {
                removeKeys(shortcuts);
            }
        };
    }, [shortcuts]);
    return useMemo(() => keys.reduce((agg, cur) => {
        if (!agg.find((a) => a.keys === cur.keys && a.ref && cur.ref)) {
            agg.push(cur);
        }
        return agg;
    }, []), [keys]);
};

export { useHotkeyState, useHotkeys };
