(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('mousetrap')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'mousetrap'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.reakeys = {}, global.react, global.Mousetrap));
}(this, (function (exports, react, Mousetrap) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var Mousetrap__default = /*#__PURE__*/_interopDefaultLegacy(Mousetrap);

  /**
   * Creates a global state singleton.
   */
  const createStateHook = () => {
      let keys = [];
      const addKeys = (nextKeys) => {
          keys = [...keys, ...nextKeys];
          nextKeys.forEach((k) => {
              var _a;
              if (!k.disabled) {
                  const mousetrap = ((_a = k.ref) === null || _a === void 0 ? void 0 : _a.current) ? Mousetrap__default['default'](k.ref.current) : Mousetrap__default['default'];
                  mousetrap.bind(k.keys, k.callback, k.action);
              }
          });
      };
      const removeKeys = (nextKeys) => {
          keys = keys.filter((k) => !nextKeys.includes(k));
          nextKeys.forEach(s => Mousetrap__default['default'].unbind(s.keys, s.action));
      };
      return () => {
          const [state, setState] = react.useState([]);
          react.useEffect(() => {
              setState(keys);
          }, []);
          return [state, addKeys, removeKeys];
      };
  };
  const useHotkeyState = createStateHook();

  const useHotkeys = (shortcuts) => {
      const [keys, addKeys, removeKeys] = useHotkeyState();
      react.useLayoutEffect(() => {
          if (shortcuts) {
              addKeys(shortcuts);
          }
          return () => {
              if (shortcuts) {
                  removeKeys(shortcuts);
              }
          };
      }, [shortcuts]);
      return react.useMemo(() => keys.reduce((agg, cur) => {
          if (!agg.find((a) => a.keys === cur.keys && a.ref && cur.ref)) {
              agg.push(cur);
          }
          return agg;
      }, []), [keys]);
  };

  exports.useHotkeyState = useHotkeyState;
  exports.useHotkeys = useHotkeys;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
